@@ -6,6 +6,9 @@ from numpy import ndarray
 def fill_missing(data: ndarray) -> ndarray:
     """ Fill missing values with means of non-nan values across rows.
 
+    Deal with missing values by assigning the mean value
+    of each bin to missing values of that bin
+
     Parameters
     ----------
     data : ndarray
@@ -17,9 +20,12 @@ def fill_missing(data: ndarray) -> ndarray:
         copy of data with missing entries filled 
     """    
 
-    # Deal with missing values by assigning the mean value
-    # of each bin to missing values of that bin
     data = data.copy()
+
+    # Set bins with nan across all cells to 0
+    data[:, np.all(np.isnan(data), axis=0)] = 0
+
+    # Mean of each bin, ignoring nan
     bin_means = np.nanmean(data, axis=0)
     bin_means = np.nan_to_num(bin_means, nan=0)
     bin_means = np.tile(bin_means, (data.shape[0], 1))
